# Definition

The data that we have is made of daily adjusted close prices for a set number of stocks over a period of 6 months. An anomaly is defined as a latest daily movement that is peculiar with regards to the historical movements of 1) this stock, 2) all others stocks, 3) the latest daily movement of all other stocks. In particular for our purpose, a big market move that affects all stocks, like a central bank interest rate decision, or macroeconomic news, should not be flagged as an anomaly. It should only detect moves that result from an idiosyncratic behavior that one or a few stocks exhibit.

# Stock Price Analysis

Classical financial theories assume that log returns of stock prices follow normal distributions. This assumption can easily be verified when analyzing the returns of one given stock.

```math
{\displaystyle r_T = log(P_T) - log(P_{T-1})}
```

A normality test is run on the log returns of a stock. For [this one](insight:YrThxo5), the test does not reject the normality hypothesis, but it does for other stocks. Given the sample sizes it is not a surprise. Furthermore, these log normal distributions often exhibit fat tails, meaning that the probability of the extreme events are higher than expected when fitting the middle of the distribution.

[Statistic worksheet on log returns](statistics_worksheet:JKHMBxQ1bV)

Additionally, stock prices are correlated to each other in multiple ways. This is the reason why returns of a portfolio or a stock are often expressed using factor models like [CAPM](https://en.wikipedia.org/wiki/Capital_asset_pricing_model) which uses the overall market as a factor or more complex ones involving sectors or subparts of the market like [the Fama-French model](https://en.wikipedia.org/wiki/Fama%E2%80%93French_three-factor_model).

In our analysis, a day's log return will be expected to follow a [Multivariate Normal Distribution](https://en.wikipedia.org/wiki/Multivariate_normal_distribution) with a covariance matrix and a mean vector estimated empirically. The covariance computation becomes heavy when the number of stocks is large, as is the case for our initial example with the S&P 500 stocks. Moreover, the anomaly detection involves the inversion of the covariance matrix, which is tricky when the number dimensions grow. To address this challenge the stocks are clustered into smaller groups and the anomaly detection is run on these more tractably sized groups.

# Stock Clustering

The aim of the clustering is twofold: reduce the size of the covariance matrices and regroup stocks that show similar patterns. 

To run this clustering, a covariance matrix is built using the historical log returns. Then a distance must be defined to assess how far stocks are from each other. A simple way of doing so is by running a [Principal Component Analysis](https://en.wikipedia.org/wiki/Principal_component_analysis), extracting the first eigenvectors and using these projections as coordinates of each stock. If we use the two first eigenvectors, we could project the stocks onto a plane, with the x axis being the first eigenvector and the y axis being the second eigenvector, as shown on the below graph.

[PC1 vs PC2 on computed_PCA_coordinates](insight:9poTQwA)

Once the stocks have been projected, a [k-means clustering algorithm](https://en.wikipedia.org/wiki/K-means_clustering) is run, which outputs cluster labels for each stock. Anomaly detection as oultined below will then occur on each cluster independently.

# Anomaly Detection

The anomaly detection algorithm implemented uses the [Mahalanobis Distance](https://en.wikipedia.org/wiki/Mahalanobis_distance), it is explained in detail in this [video](https://www.youtube.com/watch?v=JjB58InuTqM). The Mahalanobis Distance between an observation and the distribution is defined as:

```math
{\displaystyle D_M(x) = \sqrt{(x - \mu)^\mathsf{T} \mathbf{\Sigma}^{-1} (x - \mu)} }
```

Where $`x = [x_1, x_2, ..., x_n]`$ is the vector of log returns for a given day, $`\mu = [\mu_1, \mu_2, ..., \mu_n]`$ is the vector of means,  $`\Sigma`$ is the covariance matrix and $`n`$ is the number of stocks in this cluster.

This distance gives the likelihood of a return vector being an anomaly. Since we are interested in anomalies occurring at the level of individual stocks, the computation needs to be refined.

Additional Mahalanobis Distances are computed by replacing each stock's log return by its historical mean $`\hat{x}_i = [x_1, ... x_{i-1}, \mu_i, x_{i+1}, x_n]`$. And the anomaly distance is computed for each stock as:

```math
{\displaystyle D_A(i) = D_M(x) - D_M(\hat{x}_i) }
```

Hence, it can be interpreted as the contribution of each stock to the anomalousness of a given observation. When a contribution is larger than a predefined threshold, this stock is deemed to exhibit an anomalous move for this day.

